AREApplicationController.prototype = new AREViewController();
AREApplicationController.prototype.constructor = AREApplicationController;

function AREApplicationController() {
	AREViewController.prototype.constructor.call(this);
	this.view = AREView.viewWithElement($("#applicationView"));

	this.displayManager = null;
	this.startupView = null;
	this.menuView = null;
	this.nextMenuView = null;
	this.navigationBarView = null;
	this.pageNavigationBar = null;
	this.rootViewController = null;
	this.pageViewController = null;
	this.modalView = null;
	this.mapView = null;
	this.menuOpen = false;
	this.navigationBarVisible = true;
	this.categoryPageView = null;
	this.subCategoryPageView = null;
	this.advertQueuePageView = null;
	this.articlePageView = null;
	this.rootViewCover = null;
	this.legalView = null;
	this.currentLocations = null;
	this.arrowsView = null;
	this.ooeSubMenuItems = new Array();
	this.ooeTypes = new Array();

	this.currentCategoryData = null;

	this.init();
}

AREApplicationController.prototype.init = function() {
	var environment = new Environment();
	Environment.sharedInstance = environment;

	this.modalView = new AREModalView();
	AREModalView.sharedInstance = this.modalView;
	this.view.addSubview(this.modalView);

	this.displayManager = new AREDisplayManager();
	AREDisplayManager.sharedInstance = this.displayManager;

	this.startupView = new AREStartupView();
	$(this.startupView).bind(AREStartupViewEvent.STARTUP_COMPLETE, $.proxy(this.startApplication, this));
	this.view.addSubview(this.startupView);

	this.startupView.beginSetup();
}

AREApplicationController.prototype.nativePluginResultHandler = function(result) {
	//alert('nativePluginResultHandler - '+result);
	// console.log('nativePluginResultHandler: '+result);
}

AREApplicationController.prototype.nativePluginErrorHandler = function(error) {
	//alert('nativePluginErrorHandler - '+error);
	// console.log('nativePluginErrorHandler: '+error);
}

AREApplicationController.prototype.startApplication = function() {
	// ADD MENU VIEW
	this.menuView = new AREMenuView();
	this.menuView.setBackgroundColor("#464646");
	this.buildMainMenu();
	this.view.addSubview(this.menuView);

	// ADD MAIN VIEW CONTROLLER...
	this.rootViewController = new AREViewController("rootView");
	this.rootViewController.view.setBackgroundColor("#bebebe");
	this.view.addSubview(this.rootViewController.view);

	// ADD PAGE CONTROLLER...
	this.pageViewController = new AREPageViewController("pageViewController");
	$(this.pageViewController).bind(AREPageViewControllerEvent.PAGE_READY, $.proxy(this.pageReadyEvent, this));
	this.rootViewController.view.addSubview(this.pageViewController.view);
	this.updatePageViewController();

	// ADD NAVIGATION BAR...
	this.navigationBarView = new ARENavigationBarView("navigationBarView");
	this.rootViewController.view.addSubview(this.navigationBarView);

	$(this.navigationBarView.menuButton.html).click($.proxy(this.menuButtonSelected, this));
	$(this.navigationBarView.mapButton.html).click($.proxy(this.mapButtonSelected, this));

	if (ApplicationData.sharedInstance.locations.length > 0) {
		this.navigationBarView.mapButton.setHidden(false);
	} else {
		this.navigationBarView.mapButton.setHidden(true);
	}
	// if (!ApplicationData.sharedInstance.hasMaps) this.navigationBarView.mapButton.setHidden(true);

	// ADD PAGE NAVIGATION BAR...
	this.pageNavigationBar = new AREPageNavigationBarView("pageNavigationBarView");
	$(this.pageNavigationBar).bind(AREPageNavigationBarEvent.MENU_BUTTON_PRESSED, $.proxy(this.toggleMenuBar, this));
	$(this.pageNavigationBar).bind(AREPageNavigationBarEvent.HOME_BUTTON_PRESSED, $.proxy(this.homeButtonPressed, this));
	$(this.pageNavigationBar).bind(AREPageNavigationBarEvent.GRID_BUTTON_PRESSED, $.proxy(this.gridNavigationButtonPressed, this));
	$(this.pageNavigationBar).bind(AREPageNavigationBarEvent.BACK_BUTTON_PRESSED, $.proxy(this.backNavigationButtonPressed, this));
	$(this.pageNavigationBar).bind(AREPageNavigationBarEvent.NAVIGATE_LEFT, $.proxy(this.swipRightDetected, this));
	$(this.pageNavigationBar).bind(AREPageNavigationBarEvent.NAVIGATE_RIGHT, $.proxy(this.swipLeftDetected, this));
	//this.pageNavigationBar.rightNavButton.setHidden(true);
	this.pageNavigationBar.setHomeNavigationEnabled(false);
	this.pageNavigationBar.setLeftNavigationEnabled(false);
	this.pageNavigationBar.setRightNavigationEnabled(false);
	this.pageNavigationBar.setBackNavigationEnabled(false);
	this.pageNavigationBar.setGridNavigationEnabled(false);
	this.rootViewController.view.addSubview(this.pageNavigationBar);
	this.updatePageNavigationBar();

	this.categoryPageView = new ARECategoryPageView(ApplicationData.sharedInstance.categories);
	$(this.categoryPageView).bind(ARECategoryPageViewEvent.CATEGORY_SELECTED, $.proxy(this.categorySelectedEvent, this));
	this.pageViewController.pushView(this.categoryPageView);

	this.rootViewCover = new AREView("rootViewCover");
	this.rootViewController.view.addSubview(this.rootViewCover);
	$(this.rootViewCover.html).click($.proxy(this.closeMainMenu, this));

	AREDisplayManager.addUpdateFunction($.proxy(this.updatePageNavigationBar, this));
	AREDisplayManager.addUpdateFunction($.proxy(this.updatePageViewController, this));
	AREDisplayManager.addUpdateFunction($.proxy(this.updateMapView, this));

	$$(this.pageViewController.view.html).swipeLeft($.proxy(this.swipLeftDetected, this));
	$$(this.pageViewController.view.html).swipeRight($.proxy(this.swipRightDetected, this));

	this.arrowsView = new AREArrowContainerView();
	this.arrowsView.addViewClass("arrowContainer");
	this.view.addSubview(this.arrowsView);

	this.arrowsView.displayArrows(true,false,false);

	var iOS = /(iPad|iPhone|iPod)/g.test( navigator.userAgent );
	if (!iOS) {
		setTimeout($.proxy(this.addFirstView, this), 1000);
	}

	if (googleAnalytics && navigator && navigator.onLine) googleAnalytics.trackPage("application/categories/");
}

AREApplicationController.prototype.addFirstView = function() {
	this.categoryPageView.removeFromSuperview();
	this.categoryPageView = new ARECategoryPageView(ApplicationData.sharedInstance.categories);
	$(this.categoryPageView).bind(ARECategoryPageViewEvent.CATEGORY_SELECTED, $.proxy(this.categorySelectedEvent, this));
	this.pageViewController.setView(this.categoryPageView);
}

AREApplicationController.prototype.buildMainMenu = function() {
	if (this.menuView) {
		this.menuView.clearMenu();
		$(this.menuView).unbind();

		var homeBtn = this.menuView.addMenuItem("Home");
		$(homeBtn).bind(AREMainMenuEvent.MAIN_MENU_ITEM_SELECTED, $.proxy(this.menuHomeButtonPressed, this));

		for (var i=0; i<ApplicationData.sharedInstance.categories.length; i++) {
			var category = ApplicationData.sharedInstance.categories[i];
			var item = this.menuView.addMenuItem(category.heading);
			item.categoryObject = category;

			for (var s=0; s<category.subCategories.length; s++) {
				var subCategory = category.subCategories[s];
				var subItem = this.menuView.addSubMenuItemByIndex(i+1,subCategory.heading);
				subItem.subCategoryObject = subCategory;
			}
		}

		var legal = this.menuView.addMenuItem("Legal");
		//var archant = legal.addSubMenuItem("Archant");
		var terms = legal.addSubMenuItem("Terms & Conditions");

		//$(archant).bind(ARESubMenuItemEvent.SUB_MENU_ITEM_SELECTED, $.proxy(this.archantLegalSelected, this));
		$(terms).bind(ARESubMenuItemEvent.SUB_MENU_ITEM_SELECTED, $.proxy(this.termsLegalSelected, this));

		$(this.menuView).bind(AREMainMenuEvent.MENU_ITEM_SELECTED, $.proxy(this.menuItemSelectedEvent, this));
	}
}

AREApplicationController.prototype.menuHomeButtonPressed = function() {
	var menuWidth = $(window).width()*0.3;
	menuWidth = (menuWidth < 270) ? "270px" : Math.round(menuWidth).toString()+"px";

	AREAnimation.to(this.rootViewController.view.html, 0.4, {left:"0px", ease:"Back.easeIn", onComplete:$.proxy(this.menuHomeComplete, this)});

	this.menuOpen = false;
}

AREApplicationController.prototype.menuHomeComplete = function() {
	this.homeButtonPressed();
	this.resetMenuScrollView();
}

AREApplicationController.prototype.buildArticleMenu = function(s) {
	if (this.menuView) {
		this.menuView.clearMenu();
		$(this.menuView).unbind();

		var back = this.menuView.addMenuItem("Back to Category");
		back.enableLeftIcon();
		back.leftContainer.setBackground("images/leftArrow.png");
		back.leftContainer.setBackgroundColor("transparent");
		var category = this.menuView.addMenuItem(s.heading);

		for (var i=0; i<s.slides.length; i++) {
			var article = s.slides[i];
			var subItem = null;
			if (article.type == 0) {
				var itemHeading = (article.heading) ? article.heading : "Article";
				subItem = this.menuView.addSubMenuItemTo(category,itemHeading);
			}
			if (article.type == 1) {
				var itemHeading = (article.heading) ? article.heading : "Map";
				subItem = this.menuView.addSubMenuItemTo(category,itemHeading);
			}
			if (article.type == 2) {
				var itemHeading = (article.heading) ? article.heading : "Video";
				subItem = this.menuView.addSubMenuItemTo(category,itemHeading);
			}
			if (article.type == 3) {
				var itemHeading = "Information";
				subItem = this.menuView.addSubMenuItemTo(category,itemHeading);
			}
			if (article.type == 4) {
			}

			if (subItem) {
				subItem.index = i;
			}
		}

		category.expand();

		$(back.header.html).click($.proxy(this.articleMenuBackButtonSelectedEvent,this));

		$(this.menuView).bind(AREMainMenuEvent.MENU_ITEM_SELECTED, $.proxy(this.articleItemSelectedEvent, this));
	}
}

AREApplicationController.prototype.articleMenuBackButtonSelectedEvent = function() {
	if (this.menuView) {
		this.menuView.createNewMenu();
		this.buildMainMenu();
		this.menuView.animateNewMenu();
	}
}

AREApplicationController.prototype.articleItemSelectedEvent = function(e,i) {
	var index = i.index;

	var menuWidth = $(window).width()*0.3;
	menuWidth = (menuWidth < 270) ? "270px" : Math.round(menuWidth).toString()+"px";

	AREAnimation.to(this.rootViewController.view.html, 0.4, {left:"0px", ease:"Back.easeIn", onComplete:$.proxy(this.scrollToItem, this), onCompleteParams:[index]});
	this.menuOpen = false;
}

AREApplicationController.prototype.scrollToItem = function(i) {
	var index = i;

	if (this.articlePageView) {
		$(this.articlePageView.contentView.html).animate({
			scrollTop: $("#_articleContent"+index.toString()).position().top
		}, 400);
	}

	this.resetMenuScrollView();
}

AREApplicationController.prototype.archantLegalSelected = function() {
	this.legalView = new AREArchantLegalView();

	this.pageViewController.pushView(this.legalView);

	this.pageNavigationBar.setHomeNavigationEnabled(true);
	this.pageNavigationBar.setLeftNavigationEnabled(false);
	this.pageNavigationBar.setRightNavigationEnabled(false);
	this.pageNavigationBar.setBackNavigationEnabled(false);
	this.pageNavigationBar.setGridNavigationEnabled(false);

	this.categoryPageView = null;
	this.articlePageView = null;
	this.subCategoryPageView = null;
	this.advertQueuePageView = null;

	var menuWidth = $(window).width()*0.3;
	menuWidth = (menuWidth < 270) ? "270px" : Math.round(menuWidth).toString()+"px";

	AREAnimation.to(this.rootViewController.view.html, 0.4, {left:"0px", ease:"Back.easeIn", onComplete:$.proxy(this.resetMenuScrollView, this)});
	this.menuOpen = false;
}

AREApplicationController.prototype.termsLegalSelected = function() {
	this.legalView = new ARETermsView();

	this.pageViewController.pushView(this.legalView);

	this.pageNavigationBar.setHomeNavigationEnabled(true);
	this.pageNavigationBar.setLeftNavigationEnabled(false);
	this.pageNavigationBar.setRightNavigationEnabled(false);
	this.pageNavigationBar.setBackNavigationEnabled(false);
	this.pageNavigationBar.setGridNavigationEnabled(false);

	this.categoryPageView = null;
	this.articlePageView = null;
	this.subCategoryPageView = null;
	this.advertQueuePageView = null;

	var menuWidth = $(window).width()*0.3;
	menuWidth = (menuWidth < 270) ? "270px" : Math.round(menuWidth).toString()+"px";

	AREAnimation.to(this.rootViewController.view.html, 0.4, {left:"0px", ease:"Back.easeIn", onComplete:$.proxy(this.resetMenuScrollView, this)});
	this.menuOpen = false;
}

AREApplicationController.prototype.buildMapMenu = function() {
	if (this.menuView) {
		this.menuView.clearMenu();
		$(this.menuView).unbind();

		for (var i=0; i<ApplicationData.sharedInstance.categories.length; i++) {
			var category = ApplicationData.sharedInstance.categories[i];
			var item = this.menuView.addMenuItem(category.heading);
			item.hasColouredIcon(category.color);
			item.hasCheckBox(true);
			item.categoryObject = category;
			$(item).bind(ARECheckboxEvent.STATE_CHANGED, $.proxy(this.updateMarkers, this));

			for (var s=0; s<category.subCategories.length; s++) {
				var subCategory = category.subCategories[s];
				var subItem = this.menuView.addSubMenuItemByIndex(i,subCategory.heading);
				subItem.subCategoryObject = subCategory;
				subItem.locationData = ApplicationData.sharedInstance.getLocationById(subCategory.location);
			}

			if (this.currentCategoryData) {
				if (category != this.currentCategoryData) {
					item.checkBox.setChecked(false);
				}
			}
		}

		if (this.categoryPageView == null) {
			var ooeItem = this.menuView.addMenuItem("All Listings");
			ooeItem.hasColouredIcon("#ffffff");
			ooeItem.hasCheckBox(true);
			ooeItem.checkBox.setChecked(false);
			$(ooeItem).bind(ARECheckboxEvent.STATE_CHANGED, $.proxy(this.switchOoeData, this));

			this.ooeSubMenuItems = new Array();
			this.ooeTypes = new Array();

			// USE CATEGORIES 118 IN THE CURRENT CATEGORY ONLY
			for (var i = 0; i < this.currentCategoryData.json.categories118.length; i++) {
				var cat118 = this.currentCategoryData.json.categories118[i];

				var obj = ApplicationData.sharedInstance.ooeData.locations118[cat118];
				var subItem = this.menuView.addSubMenuItemTo(ooeItem,cat118,true);
				subItem.hasCheckBox(true);
				subItem.checkBox.setChecked(false);
				$(subItem).bind(ARESubMenuItemEvent.SUB_MENU_ITEM_SELECTED, $.proxy(this.showOoeData, this));
				this.ooeSubMenuItems.push(subItem);
			}

			// USE ALL CATEGORIES 118 IN THE APPLICATION
			// for (var cat118 in ApplicationData.sharedInstance.ooeData.locations118) {
			//
			// 	var obj = ApplicationData.sharedInstance.ooeData.locations118[cat118];
			// 	var subItem = this.menuView.addSubMenuItemTo(ooeItem,cat118,true);
			// 	subItem.hasCheckBox(true);
			// 	subItem.checkBox.setChecked(false);
			// 	$(subItem).bind(ARESubMenuItemEvent.SUB_MENU_ITEM_SELECTED, $.proxy(this.showOoeData, this));
			// 	this.ooeSubMenuItems.push(subItem);
			// }
		}

		/*var legal = this.menuView.addMenuItem("Legal");
		var archant = legal.addSubMenuItem("Archant");
		var terms = legal.addSubMenuItem("Terms & Conditions");*/

		$(this.menuView).bind(AREMainMenuEvent.MENU_ITEM_SELECTED, $.proxy(this.mapMenuItemSelectedEvent, this));
	}
}

AREApplicationController.prototype.showOoeData = function(e,s) {
	if (s.checkBox.checked) {
		this.ooeTypes.push(s.itemTitle);
	} else {
		for (var i=0; i<this.ooeTypes.length; i++) {
			if (this.ooeTypes[i] == s.itemTitle) {
				this.ooeTypes.splice(i, 1);
			}
		}
	}

	if (this.mapView) {
		this.mapView.displayOoeData(this.ooeTypes);
	}
}

AREApplicationController.prototype.switchOoeData = function(e,s) {
	if (this.mapView) {
		var bool = s.checkBox.checked;
		if (bool) {
			this.mapView.displayOoeData();
		} else {
			this.mapView.clearOoeData();
		}

		for (var i=0; i<this.ooeSubMenuItems.length; i++) {
			var smi = this.ooeSubMenuItems[i];
			smi.checkBox.setChecked(bool);
		}
	}
}

AREApplicationController.prototype.updateMarkers = function(e, s) {
	// // console.log("updateMarkers with s: "); // // console.log(s);
	if (this.mapView) {
		//this.mapView.clearAllMarkers();
		var subMenuItems = s.subMenuItems;
		var category = s.categoryObject;
		var add = s.checkBox.checked;
		/*for (var i=0; i<subMenuItems.length; i++) {
			var id = subMenuItems[i].subCategoryObject.location;
			if (add) {
				this.mapView.showMarkerWithId(id);
			} else {
				this.mapView.hideMarkerWithId(id);
			}
		}*/
		for (var i=0; i<category.locations.length; i++) {
			var id = category.locations[i];
			if (add) {
				this.mapView.showMarkerWithId(id);
			} else {
				this.mapView.hideMarkerWithId(id);
			}
		}
	}
}

AREApplicationController.prototype.backNavigationButtonPressed = function() {
	if (this.subCategoryPageView) {
		// console.log("Back navigation pressed from SUB-CATEGORY list");
		this.homeButtonPressed();
	}

	if (this.advertQueuePageView) {
		// console.log("Back navigation pressed from GRID list");
		var currentCategory = this.advertQueuePageView.subCategory.parentCategory;
		this.categorySelectedEvent(null, currentCategory, true);
	}

	// if (this.articlePageView) {
	// 	var currentSubCategory = this.articlePageView.advertQueue.parentSubCategory;
	// 	this.subCategorySelectedEvent(null, currentSubCategory, true);
	// }

	if (this.articlePageView) {
		// console.log("Back navigation pressed from ADVERT QUEUE");
		var currentCategory = this.articlePageView.advertQueue.parentSubCategory.parentCategory;
		this.categorySelectedEvent(null, currentCategory, true);
	}
}

AREApplicationController.prototype.gridNavigationButtonPressed = function() {
	if (this.subCategoryPageView) {
		// console.log("Grid navigation pressed from SUB-CATEGORY list");
	}

	if (this.advertQueuePageView) {
		// console.log("Grid navigation pressed from GRID list");
	}

	// if (this.articlePageView) {
	// 	var currentSubCategory = this.articlePageView.advertQueue.parentSubCategory;
	// 	this.subCategorySelectedEvent(null, currentSubCategory, true);
	// }

	if (this.articlePageView) {
		// console.log("Grid navigation pressed from ADVERT QUEUE");
		var currentSubCategory = this.articlePageView.advertQueue.parentSubCategory;
		this.gridSelectedEvent(null, currentSubCategory, true);
	}
}

AREApplicationController.prototype.swipLeftDetected = function() {
	if (theAppIsBusy == true) return false;

	if (this.subCategoryPageView) {

		theAppIsBusy = true;
		setTimeout(function(){
			theAppIsBusy = false;
		}, 1000);

		var currentIndex = this.subCategoryPageView.category.index;
		if (currentIndex < ApplicationData.sharedInstance.categories.length-1) {
			currentIndex++;
			var nextCategory = ApplicationData.sharedInstance.categories[currentIndex];
			this.categorySelectedEvent(null,nextCategory);
		}
	}

	if (this.advertQueuePageView) {

		theAppIsBusy = true;
		setTimeout(function(){
			theAppIsBusy = false;
		}, 1000);

		var subCategories = this.advertQueuePageView.subCategory.parentCategory.subCategories;
		var currentIndex = this.advertQueuePageView.subCategory.index;
		if (currentIndex < subCategories.length-1) {
			currentIndex++;
			var nextCategory = subCategories[currentIndex];
			this.gridSelectedEvent(null, nextCategory);
		}
	}

	if (this.articlePageView) {

		theAppIsBusy = true;
		setTimeout(function(){
			theAppIsBusy = false;
		}, 1000);

		var currentCategory = this.articlePageView.advertQueue.parentSubCategory;
		var currentIndex = this.articlePageView.advertQueue.index;
		if (currentIndex < currentCategory.advertQueues.length-1) {
			currentIndex++;
			var nextSubCategory = currentCategory.advertQueues[currentIndex];
			this.articleSelectedEvent(null,nextSubCategory);
		}
	}
}

AREApplicationController.prototype.swipRightDetected = function() {
	if (theAppIsBusy == true) return false;

	if (this.subCategoryPageView) {

		theAppIsBusy = true;
		setTimeout(function(){
			theAppIsBusy = false;
		}, 1000);

		var currentIndex = this.subCategoryPageView.category.index;
		if (currentIndex > 0) {
			currentIndex--;
			var nextCategory = ApplicationData.sharedInstance.categories[currentIndex];
			this.categorySelectedEvent(null,nextCategory,true);
		}
	}

	if (this.advertQueuePageView) {

		theAppIsBusy = true;
		setTimeout(function(){
			theAppIsBusy = false;
		}, 1000);

		var subCategories = this.advertQueuePageView.subCategory.parentCategory.subCategories;
		var currentIndex = this.advertQueuePageView.subCategory.index;
		if (currentIndex > 0) {
			currentIndex--;
			var nextCategory = subCategories[currentIndex];
			this.gridSelectedEvent(null, nextCategory,true);
		}
	}

	if (this.articlePageView) {

		theAppIsBusy = true;
		setTimeout(function(){
			theAppIsBusy = false;
		}, 1000);

		var currentCategory = this.articlePageView.advertQueue.parentSubCategory;
		var currentIndex = this.articlePageView.advertQueue.index;
		if (currentIndex > 0) {
			currentIndex--;
			var nextSubCategory = currentCategory.advertQueues[currentIndex];
			this.articleSelectedEvent(null,nextSubCategory,true);
		}
	}
}

AREApplicationController.prototype.pageReadyEvent = function(v) {
	if (this.startupView) {
		this.startupView.removeFromSuperview();
		this.startupView = null;
	} else {
		AREModalView.sharedInstance.removeCurrentView();
	}
}

AREApplicationController.prototype.categorySelectedEvent = function(e, c, p) {

	// console.log("CATEGORY_SELECTED has been triggered with c = "); console.log(c);

	// AREModalView.sharedInstance.displayView(new ARESpinnerView());

	this.currentLocations = c.locations;
	if (this.currentLocations.length > 0) {
		this.navigationBarView.mapButton.setHidden(false);
	} else {
		this.navigationBarView.mapButton.setHidden(true);
	}

	// if (!ApplicationData.sharedInstance.hasMaps) this.navigationBarView.mapButton.setHidden(true);

	this.currentCategoryData = c;

	this.subCategoryPageView = new ARESubCategoryPageView(c);
	$(this.subCategoryPageView).bind(ARESubCategoryPageViewEvent.SUB_CATEGORY_SELECTED, $.proxy(this.subCategorySelectedEvent, this));
	if (p) {
		this.pageViewController.popView(this.subCategoryPageView);
	} else {
		this.pageViewController.pushView(this.subCategoryPageView);
	}

	this.pageNavigationBar.setHomeNavigationEnabled(true);
	this.pageNavigationBar.setBackNavigationEnabled(true);
	this.pageNavigationBar.setLeftNavigationEnabled(true);
	this.pageNavigationBar.setRightNavigationEnabled(true);
	this.pageNavigationBar.setGridNavigationEnabled(false);

	if ( (c.index > 0) && (c.index < c.parentApplication.categories.length-1) ) {
		// Not the first or the last element
		this.arrowsView.displayArrows(false,true,true);
	} else if (0 == c.parentApplication.categories.length-1) {
		// First and last element (only 1 exists)
		this.arrowsView.displayArrows(false,false,false);
		this.pageNavigationBar.setLeftNavigationEnabled(false);
		this.pageNavigationBar.setRightNavigationEnabled(false);
	} else {
		if (c.index <= 0) {
			// First element
			this.arrowsView.displayArrows(false,false,true);
			this.pageNavigationBar.setLeftNavigationEnabled(false);
		}
		if (c.index >= c.parentApplication.categories.length-1) {
			// Last element
			this.arrowsView.displayArrows(false,true,false);
			this.pageNavigationBar.setRightNavigationEnabled(false);
		}
	}

	this.categoryPageView = null;
	this.articlePageView = null;
	this.advertQueuePageView = null;
	this.legalView = null;
}

AREApplicationController.prototype.subCategorySelectedEvent = function(e, s, p) {

	// console.log("SUB_CATEGORY_SELECTED has been triggered with s = "); // // console.log(s);

	if (s.parentCategory) this.currentCategoryData = s.parentCategory;

	if ( s.advertQueues ) {
		var aq = s.advertQueues[0];
	} else {
		var aq = s;
		if (s.parentSubCategory) {
			if (s.parentSubCategory.parentCategory) this.currentCategoryData = s.parentSubCategory.parentCategory;
		}
	}

	// AREModalView.sharedInstance.displayView(new ARESpinnerView());

	this.articlePageView = new AREArticlePageView(aq);
	$(this.articlePageView).bind(AREPageEvent.EXPAND_BUTTON_PRESSED, $.proxy(this.mapButtonSelected, this));
	// $(this.articlePageView).bind(AREPageEvent.VIEW_ARTICLE, $.proxy(this.viewArticleFromMapContent, this));
	if (p) {
		this.pageViewController.popView(this.articlePageView);
	} else {
		this.pageViewController.pushView(this.articlePageView);
	}

	this.pageNavigationBar.setHomeNavigationEnabled(true);
	this.pageNavigationBar.setBackNavigationEnabled(true);
	this.pageNavigationBar.setLeftNavigationEnabled(true);
	this.pageNavigationBar.setRightNavigationEnabled(true);
	this.pageNavigationBar.setGridNavigationEnabled(true);

	if ( (aq.index > 0) && (aq.index < aq.parentSubCategory.advertQueues.length-1) ) {
		// Not the first or the last element
		this.arrowsView.displayArrows(true,true,true);
	} else if (0 == aq.parentSubCategory.advertQueues.length-1) {
		// First and last element (only 1 exists)
		this.arrowsView.displayArrows(true,false,false);
		this.pageNavigationBar.setLeftNavigationEnabled(false);
		this.pageNavigationBar.setRightNavigationEnabled(false);
	} else {
		if (aq.index <= 0) {
			// First element
			this.arrowsView.displayArrows(true,false,true);
			this.pageNavigationBar.setLeftNavigationEnabled(false);
		}
		if (aq.index >= aq.parentSubCategory.advertQueues.length-1) {
			// Last element
			this.arrowsView.displayArrows(true,true,false);
			this.pageNavigationBar.setRightNavigationEnabled(false);
		}
	}

	this.categoryPageView = null;
	this.subCategoryPageView = null;
	this.advertQueuePageView = null;
	this.legalView = null;

	this.buildArticleMenu(aq);

	// var iOS = /(iPad|iPhone|iPod)/g.test( navigator.userAgent );
	// if (!iOS) {
		setTimeout($.proxy(this.addFirstArticleView, this), 1000);
	// }
}

AREApplicationController.prototype.articleSelectedEvent = function(e, aq, p) {

	// // console.log("ADVERT QUEUE SELECTED");
	if (aq.parentCategory) this.currentCategoryData = aq.parentCategory;
	if (aq.parentSubCategory) {
		if (aq.parentSubCategory.parentCategory) this.currentCategoryData = aq.parentSubCategory.parentCategory;
	}

	// AREModalView.sharedInstance.displayView(new ARESpinnerView());

	this.articlePageView = new AREArticlePageView(aq);
	$(this.articlePageView).bind(AREPageEvent.EXPAND_BUTTON_PRESSED, $.proxy(this.mapButtonSelected, this));
	// $(this.articlePageView).bind(AREPageEvent.VIEW_ARTICLE, $.proxy(this.viewArticleFromMapContent, this));
	if (p) {
		this.pageViewController.popView(this.articlePageView);
	} else {
		this.pageViewController.pushView(this.articlePageView);
	}

	this.pageNavigationBar.setHomeNavigationEnabled(true);
	this.pageNavigationBar.setBackNavigationEnabled(true);
	this.pageNavigationBar.setLeftNavigationEnabled(true);
	this.pageNavigationBar.setRightNavigationEnabled(true);
	this.pageNavigationBar.setGridNavigationEnabled(true);

	if ( (aq.index > 0) && (aq.index < aq.parentSubCategory.advertQueues.length-1) ) {
		// Not the first or the last element
		this.arrowsView.displayArrows(true,true,true);
	} else if (0 == aq.parentSubCategory.advertQueues.length-1) {
		// First and last element (only 1 exists)
		this.arrowsView.displayArrows(true,false,false);
		this.pageNavigationBar.setLeftNavigationEnabled(false);
		this.pageNavigationBar.setRightNavigationEnabled(false);
	} else {
		if (aq.index <= 0) {
			// First element
			this.arrowsView.displayArrows(true,false,true);
			this.pageNavigationBar.setLeftNavigationEnabled(false);
		}
		if (aq.index >= aq.parentSubCategory.advertQueues.length-1) {
			// Last element
			this.arrowsView.displayArrows(true,true,false);
			this.pageNavigationBar.setRightNavigationEnabled(false);
		}
	}

	this.categoryPageView = null;
	this.subCategoryPageView = null;
	this.advertQueuePageView = null;
	this.legalView = null;

	this.buildArticleMenu(aq);

	// var iOS = /(iPad|iPhone|iPod)/g.test( navigator.userAgent );
	// if (!iOS) {
		setTimeout($.proxy(this.addFirstArticleView, this), 1000);
	// }
}

AREApplicationController.prototype.addFirstArticleView = function() {
	// Destroy the advert queue and create again, to avoid Android content cropping issues
	// var aq = this.articlePageView.advertQueue;
	// this.articlePageView.removeFromSuperview(true);
	// this.articlePageView = new AREArticlePageView(aq);
	// $(this.articlePageView).bind(AREPageEvent.EXPAND_BUTTON_PRESSED, $.proxy(this.mapButtonSelected, this));
	// $(this.articlePageView).bind(AREPageEvent.VIEW_ARTICLE, $.proxy(this.viewArticleFromMapContent, this));
	// this.pageViewController.setView(this.articlePageView);
	this.articlePageView.viewAdded();
}

AREApplicationController.prototype.gridSelectedEvent = function(e, s, p) {

	// console.log("GRID SELECTED has been triggered with s = "); console.log(s);
	// AREModalView.sharedInstance.displayView(new ARESpinnerView());

	this.advertQueuePageView = new AREAdvertQueuePageView(s);
	$(this.advertQueuePageView).bind(AREAdvertQueuePageViewEvent.ADVERT_QUEUE_SELECTED, $.proxy(this.articleSelectedEvent, this));
	if (p) {
		this.pageViewController.popView(this.advertQueuePageView);
	} else {
		this.pageViewController.pushView(this.advertQueuePageView);
	}

	this.pageNavigationBar.setHomeNavigationEnabled(true);
	this.pageNavigationBar.setBackNavigationEnabled(true);
	this.pageNavigationBar.setLeftNavigationEnabled(true);
	this.pageNavigationBar.setRightNavigationEnabled(true);
	this.pageNavigationBar.setGridNavigationEnabled(false);

	if ( (s.index > 0) && (s.index < s.parentCategory.subCategories.length-1) ) {
		// Not the first or the last element
		this.arrowsView.displayArrows(false,true,true);
	} else if (0 == s.parentCategory.subCategories.length-1) {
		// First and last element (only 1 exists)
		this.arrowsView.displayArrows(false,false,false);
		this.pageNavigationBar.setLeftNavigationEnabled(false);
		this.pageNavigationBar.setRightNavigationEnabled(false);
	} else {
		if (s.index <= 0) {
			// First element
			this.arrowsView.displayArrows(false,false,true);
			this.pageNavigationBar.setLeftNavigationEnabled(false);
		}
		if (s.index >= s.parentCategory.subCategories.length-1) {
			// Last element
			this.arrowsView.displayArrows(false,true,false);
			this.pageNavigationBar.setRightNavigationEnabled(false);
		}
	}

	this.categoryPageView = null;
	this.subCategoryPageView = null;
	this.articlePageView = null;
	this.legalView = null;

	this.buildMainMenu();
}

AREApplicationController.prototype.homeButtonPressed = function() {
	// AREModalView.sharedInstance.displayView(new ARESpinnerView());

	this.pageNavigationBar.setHomeNavigationEnabled(false);
	this.pageNavigationBar.setBackNavigationEnabled(false);
	this.pageNavigationBar.setLeftNavigationEnabled(false);
	this.pageNavigationBar.setRightNavigationEnabled(false);
	this.pageNavigationBar.setGridNavigationEnabled(false);

	if (ApplicationData.sharedInstance.locations.length > 0) {
		this.navigationBarView.mapButton.setHidden(false);
	} else {
		this.navigationBarView.mapButton.setHidden(true);
	}
	// if (!ApplicationData.sharedInstance.hasMaps) this.navigationBarView.mapButton.setHidden(true);

	this.categoryPageView = new ARECategoryPageView(ApplicationData.sharedInstance.categories);
	$(this.categoryPageView).bind(ARECategoryPageViewEvent.CATEGORY_SELECTED, $.proxy(this.categorySelectedEvent, this));
	this.pageViewController.popView(this.categoryPageView);

	this.arrowsView.displayArrows(true,false,false);

	this.currentCategoryData = null;
	this.currentLocations = null;
	this.subCategoryPageView = null;
	this.articlePageView = null;
	this.advertQueuePageView = null;
	this.legalView = null;

	this.buildMainMenu();

	if (googleAnalytics && navigator && navigator.onLine) googleAnalytics.trackPage("application/categories/");
}

AREApplicationController.prototype.hideNavigationBar = function() {
	if (this.navigationBarVisible) {
		var navBarHeight = this.navigationBarView.height()+1;
		AREAnimation.to(this.navigationBarView.html, 0.3, {top:(0-navBarHeight).toString()+"px", ease:"Expo.easeInOut", onUpdate:function() { AREDisplayManager.update(); }});
		//this.navigationBarView.y((0-navBarHeight).toString()+"px");
		this.navigationBarVisible = !this.navigationBarVisible;
		AREDisplayManager.update();
	}
}

AREApplicationController.prototype.showNavigationBar = function() {
	if (!this.navigationBarVisible) {
		var navBarHeight = this.navigationBarView.height();
		AREAnimation.to(this.navigationBarView.html, 0.3, {top:"0px", ease:"Expo.easeInOut", onUpdate:function() { AREDisplayManager.update(); }});
		//this.navigationBarView.y("0px");
		this.navigationBarVisible = !this.navigationBarVisible;
		AREDisplayManager.update();
	}
}

AREApplicationController.prototype.toggleMenuBar = function() {
	if (this.navigationBarVisible) {
		this.hideNavigationBar();
	} else {
		this.showNavigationBar();
	}
}

AREApplicationController.prototype.updatePageNavigationBar = function() {
	if (this.navigationBarView) this.pageNavigationBar.y((this.navigationBarView.y()+this.navigationBarView.height()).toString()+"px");
}

AREApplicationController.prototype.updateMapView = function() {
	// console.log("Called updateMapView");
	if (this.mapView) {
		if (this.navigationBarView) {
			this.mapView.y((this.navigationBarView.height()+this.navigationBarView.y()).toString()+"px");
			this.mapView.height(($(window).height()-(this.navigationBarView.height()+this.navigationBarView.y())).toString()+"px");
			console.log(this.mapView);
		}
	}
}

AREApplicationController.prototype.updatePageViewController = function() {
	var offset = 0;
	if (this.navigationBarView) offset = this.navigationBarView.y()+this.navigationBarView.height();
	if (this.pageNavigationBar) offset = this.pageNavigationBar.y()+this.pageNavigationBar.height();
	this.pageViewController.view.y(offset.toString()+"px");
	this.pageViewController.view.height(($(window).height()-offset).toString()+"px");
}

AREApplicationController.prototype.menuItemSelectedEvent = function(e,i) {
	var subCategory = i.subCategoryObject;

	var menuWidth = $(window).width()*0.3;
	menuWidth = (menuWidth < 270) ? "270px" : Math.round(menuWidth).toString()+"px";

	AREAnimation.to(this.rootViewController.view.html, 0.4, {left:"0px", ease:"Back.easeIn", onComplete:$.proxy(this.updateWithArticle, this), onCompleteParams:[subCategory]});
	this.menuOpen = false;
}

AREApplicationController.prototype.mapMenuItemSelectedEvent = function(e,i) {

	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {

		if (this.mapView && i.parentItem.itemHasCheckbox) {
			if (i.parentItem.checkBox.checked) {
				var locations = i.subCategoryObject.parentCategory.locations;
				var bounds = new google.maps.LatLngBounds();

				for (var i=0; i<locations.length; i++) {
					var loc = ApplicationData.sharedInstance.getLocationById(locations[i]);
					var lngLat = new google.maps.LatLng(loc.latitude, loc.longitude);
					bounds.extend(lngLat);
				}

				this.mapView.map.fitBounds(bounds);
			}
		}
	} else {
		console.log("No Google no fun");

		var n = new ARENotificationView({heading:"CONNECTION ISSUE",body:"You are not connected to the internet, maps are not available."});
		AREModalView.sharedInstance.displayNotification(n);
	}
}

AREApplicationController.prototype.updateWithArticle = function(s) {
	if (this.menuView) {
		this.menuView.scrollView.html.scrollTop = 0;
	}
	this.rootViewCover.setHidden(true);

	this.subCategorySelectedEvent(null,s);
}

AREApplicationController.prototype.menuButtonSelected = function() {

	var menuWidth = $(window).width()*0.3;
	menuWidth = (menuWidth < 270) ? "270px" : Math.round(menuWidth).toString()+"px";

	if (this.menuOpen) {
		AREAnimation.to(this.rootViewController.view.html, 0.4, {left:"0px", ease:"Back.easeIn", onComplete:$.proxy(this.resetMenuScrollView, this)});
	} else {
		this.rootViewCover.setHidden(false);
		AREAnimation.to(this.rootViewController.view.html, 0.4, {left:menuWidth, ease:"Back.easeOut"});
	}

	this.menuOpen = !this.menuOpen;
}

AREApplicationController.prototype.closeMainMenu = function() {
	var menuWidth = $(window).width()*0.3;
	menuWidth = (menuWidth < 270) ? "270px" : Math.round(menuWidth).toString()+"px";

	AREAnimation.to(this.rootViewController.view.html, 0.4, {left:"0px", ease:"Back.easeIn", onComplete:$.proxy(this.resetMenuScrollView, this)});
	this.menuOpen = false;
}

AREApplicationController.prototype.mapButtonSelected = function(e,l,locs) {
	var coords = null;
	var mapLocations = null;

	console.log("mapButtonSelected");

	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {

		if (l) {
			loc = ApplicationData.sharedInstance.getLocationById(l);

			coords = new google.maps.LatLng(loc.latitude, loc.longitude);

			//coords = l;
		}
		if (locs) {
			mapLocations = locs;
		}
		if (this.mapView) {
			AREAnimation.to(this.mapView.html, 0.6, {top:$(window).height(), ease:"Expo.easeInOut", onComplete:$.proxy(this.removeCurrentMapView, this)});
			if (this.articlePageView) {
				this.buildArticleMenu(this.articlePageView.advertQueue);
			} else {
				this.buildMainMenu();
			}
		} else {
			AREModalView.sharedInstance.displayView(new ARESpinnerView());

			var locationList = (mapLocations) ? mapLocations : this.currentLocations;

			if (this.subCategoryPageView) {
				locationList = this.subCategoryPageView.category.locations;
			}

			// console.log("Calling AREMapPageView...");
			this.mapView = new AREMapPageView({coords:coords, locations:locationList, currentCategoryData:this.currentCategoryData});
			// console.log("...Finished AREMapPageView");

			$(this.mapView).bind(AREMapPageViewEvent.MAP_LOADED, $.proxy(this.animateMapPageView, this));
			$(this.mapView).bind(AREMapPageViewEvent.FILTER_BUTTON_SELECTED, $.proxy(this.showFilterMenu, this));
			$(this.mapView).bind(AREMapPageViewEvent.CLOSE_BUTTON_SELECTED, $.proxy(this.closeMapView, this));
			$(this.mapView).bind(AREMapPageViewEvent.VIEW_MORE_SELECTED, $.proxy(this.viewArticleFromMap, this));
			this.rootViewController.view.addSubview(this.mapView);
			this.updateMapView();
			this.mapView.y($(window).height());

			this.buildMapMenu();
		}
	} else {
		console.log("No Google no fun");

		var n = new ARENotificationView({heading:"CONNECTION ISSUE",body:"You are not connected to the internet, maps are not available."});
		AREModalView.sharedInstance.displayNotification(n);
	}
}

AREApplicationController.prototype.viewArticleFromMapContent = function(e,d) {
	this.subCategorySelectedEvent(null,d);
}

AREApplicationController.prototype.viewArticleFromMap = function(e,d) {
	this.mapButtonSelected();
	this.subCategorySelectedEvent(null,d);
}

AREApplicationController.prototype.closeMapView = function() {
	this.mapButtonSelected();			// // console.log("closeMapView");
}

AREApplicationController.prototype.showFilterMenu = function() {
	this.menuButtonSelected();			// // console.log("showFilterMenu");
}

AREApplicationController.prototype.animateMapPageView = function() {
	AREModalView.sharedInstance.removeCurrentView();
	AREAnimation.to(this.mapView.html, 0.6, {top:this.navigationBarView.height()+this.navigationBarView.y(), ease:"Expo.easeInOut"});

	// // console.log("animateMapPageView");
}

AREApplicationController.prototype.removeCurrentMapView = function() {
	if (this.mapView) {
		this.mapView.removeFromSuperview();
		this.mapView = null;

		// // console.log("removeCurrentMapView");
	}
}

AREApplicationController.prototype.resetMenuScrollView = function() {
	if (this.menuView) {
		this.menuView.scrollView.html.scrollTop = 0;
	}
	this.rootViewCover.setHidden(true);

	if (this.articlePageView && !this.mapView) {
		this.buildArticleMenu(this.articlePageView.advertQueue);
	}
}