var ApplicationData = function() {
	this.json = null;
	this.categories = new Array();
	this.locations = new Array();
	this.articles = new Array();
	this.localUrl = null;
	this.ooeUrl = null;
	this.ooeData = null;
	this.hasMaps = (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) ? true : false;

	this.init();
}

ApplicationData.prototype.init = function() {
	var jsonLoader = $.getJSON("json/settings.json"+"?"+new Date().getTime());
	jsonLoader.done($.proxy(this.settingsLoaded, this));
}

ApplicationData.prototype.settingsLoaded = function(data) {
	var jsonUrl = data.feed;
	this.localUrl = data.local;
	this.ooeUrl = data.ooe;

	var jsonLoader = $.getJSON(jsonUrl+"?"+new Date().getTime());
	jsonLoader.done($.proxy(this.loadOoeData, this));
	jsonLoader.error($.proxy(this.loadLocalData, this));
}

ApplicationData.prototype.loadLocalData = function() {
	if (fileSystem) {
		fileSystem.root.getFile("data.json", {create: false, exclusive: false}, $.proxy(this.fileFound, this), $.proxy(this.fileNotFound, this));
	} else {
		var n = new ARENotificationView({heading:"FILE SYSTEM ERROR",body:"Your device does not have a file system accessable to the application."});
		AREModalView.sharedInstance.displayNotification(n);

		if (this.localUrl) {
			var jsonLoader = $.getJSON(this.localUrl+"?"+new Date().getTime());
			jsonLoader.done($.proxy(this.skipOoeData, this));
			jsonLoader.error($.proxy(this.errorLoadingData, this));
		}
	}
}

ApplicationData.prototype.fileFound = function(fileEntry) {
	fileEntry.file($.proxy(this.readFile, this), null);
}

ApplicationData.prototype.readFile = function(file) {
	var reader = new FileReader();
        reader.onloadend = $.proxy(this.fileRead, this);
        reader.readAsText(file);
}

ApplicationData.prototype.fileRead = function(evt) {
	this.json = $.parseJSON(evt.target.result);
	this.dataLoaded(null);
}

ApplicationData.prototype.fileNotFound = function() {
	if (this.localUrl) {
		var jsonLoader = $.getJSON(this.localUrl+"?"+new Date().getTime());
		jsonLoader.done($.proxy(this.skipOoeData, this));
		jsonLoader.error($.proxy(this.errorLoadingData, this));
	}
}

ApplicationData.prototype.skipOoeData = function(data) {
	this.json = data;
	this.dataLoaded(null);
}

ApplicationData.prototype.loadOoeData = function(data) {
	this.json = data;

	if (fileSystem) {
		fileSystem.root.getFile("data.json", {create: true, exclusive: false}, $.proxy(this.fileCreated, this), $.proxy(this.fileError, this));
	}
	// Comment out because in Windows Phone 8 it is not detected and it always pops up the alert message
	// else {
		// alert("No file system");
	// }

	var jsonLoader = $.getJSON(this.ooeUrl+"?"+new Date().getTime());
	jsonLoader.done($.proxy(this.dataLoaded, this));
	jsonLoader.error($.proxy(this.loadLocalData, this));
}

ApplicationData.prototype.fileCreated = function(fileEntry) {
	fileEntry.createWriter($.proxy(this.writeFile, this), null);
}

ApplicationData.prototype.fileError = function(e) {
	alert(e);
}

ApplicationData.prototype.writeFile = function(fileWriter) {
	fileWriter.onwriteend = function(e) {
		//alert('Write completed.');
	};

	fileWriter.onerror = function(e) {
		//alert('Write failed');
	};

	fileWriter.write(JSON.stringify(this.json));
}

ApplicationData.prototype.errorLoadingData = function() {
	var n = new ARENotificationView({heading:"CONNECTION ISSUE",body:"There was an error loading data. Please ensure you have an internet connection avaialable and restart the app."});
	AREModalView.sharedInstance.displayNotification(n);
}

ApplicationData.prototype.dataLoaded = function(data) {
	this.ooeData = data;

	for (var l in this.json.application.locations) {
		// console.log("pushing location");
		var location = new Location(this.json.application.locations[l]);
		this.locations.push(location);
	}

	for (var a in this.json.application.advertQueues) {
		/*var article = new Article(this.json.application.advertQueues[a].slides);
		article.id = this.json.application.advertQueues[a].id;
		this.articles.push(article);*/
		var articleQueue = new ArticleQueue(this.json.application.advertQueues[a]);
		this.articles.push(articleQueue);
	}

	for (var i=0; i<this.json.application.categories.length; i++) {
		var category = new Category(this.json.application.categories[i]);
		category.index = i;
		category.parentApplication = this;
		this.categories.push(category);
	}

	$(this).trigger(ApplicationDataEvent.DATA_LOADED);
}

ApplicationData.prototype.getArticleQueueById = function(id) {
	var article = null;

	for (var i=0; i<this.articles.length; i++) {
		if (this.articles[i].id == id) {
			article = this.articles[i];
			break;
		}
	}

	return article;
}

ApplicationData.prototype.getLocationById = function(id) {
	var location = null;

	// console.log("getLocationById: ");	// console.log(id);
	// console.log("this: ");	// console.log(this);

	for (var i=0; i<this.locations.length; i++) {
		if (this.locations[i].id == id) {
			location = this.locations[i];
			break;
		}
	}

	return location;
}

ApplicationData.sharedInstance = null;