AREMapContentView.prototype = new AREContentView();
AREMapContentView.prototype.constructor = AREMapContentView;

function AREMapContentView(obj, aQ) {
	this.dataObject = obj;
	this.navigationBar = null;
	this.map = null;
	this.mapContainer = null;
	this.expandButton = null;
	this.coords = null;
	this.advertQueue = aQ;

	this.markers = new Array();
	this.popup = null;
	this.currentPopupId = null;

	AREContentView.prototype.constructor.call(this);
}

AREMapContentView.prototype.drawView = function() {
	AREContentView.prototype.drawView.call(this);
	this.addViewClass("mapContentView");

	this.mapContainer = new AREView();
	this.mapContainer.addViewClass("mapContainer");

	this.navigationBar = new AREView();
	this.navigationBar.addViewClass("mapContentNavigationBar");
	this.addSubview(this.navigationBar);

	this.expandButton = new AREButton();
	this.expandButton.addViewClass("mapButtonRight");
	var icon = new AREView();
	icon.addViewClass("expandIcon");
	this.expandButton.addSubview(icon);
	this.navigationBar.addSubview(this.expandButton);
	$(this.expandButton.html).click($.proxy(this.expandButtonPressed, this));

	this.addSubview(this.mapContainer);

	setTimeout($.proxy(this.getLocation, this), 200);
}

AREMapContentView.prototype.fixMapMarkersNotClickable = function() {
	$('.gm-style').find('div[style*="z-index: 2"]').css('z-index','0');
}

AREMapContentView.prototype.getLocation = function() {
	if (navigator) {
		if (navigator.geolocation) navigator.geolocation.getCurrentPosition($.proxy(this.assignLocation, this), $.proxy(this.locationError, this));
	}
}

AREMapContentView.prototype.locationError = function() {
	// console.log("Geolocation Error");
}

AREMapContentView.prototype.assignLocation = function(position) {
	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {
		// console.log("Getting user position");
		User.sharedInstance.location = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
	}
}

AREMapContentView.prototype.expandButtonPressed = function() {
	var locations = (this.advertQueue) ? this.advertQueue.locations : null;

	/*if (this.advertQueue) {
		for (var i=0; i<this.advertQueue.locations.length; i++) {
			locations.push(ApplicationData.sharedInstance.getLocationById(parseInt(this.advertQueue.locations[i])));
		}
	}*/
	// console.log("expandButtonPressed with locations: ");	console.log(locations);
	//$(this).trigger(AREMapContentViewEvent.EXPAND_BUTTON_PRESSED, [null, locations]);
	$(this).trigger(AREMapContentViewEvent.EXPAND_BUTTON_PRESSED, locations);

	//  It's actually passing only the first location for some reason:
	//  The handler for this event, that is actually creating the map,
	//  is in AREApplicationController.js - mapButtonSelected
}

AREMapContentView.prototype.viewAdded = function() {
	AREContentView.prototype.viewAdded.call(this);

	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {
		var locations = new Array();

		if (this.advertQueue) {
			for (var i=0; i<this.advertQueue.locations.length; i++) {
				locations.push(ApplicationData.sharedInstance.getLocationById(parseInt(this.advertQueue.locations[i])));
			}

			if (locations[0]) {
				this.coords = new google.maps.LatLng(locations[0].latitude, locations[0].longitude);
			} else {
				this.coords = new google.maps.LatLng(0, 0);
			}
		}

		var mapStyles =[{
			featureType: "poi",
			elementType: "labels",
			stylers: [{ visibility: "off" }]
		}];
		var mapOptions = { zoom: 10, center: this.coords, mapTypeId: google.maps.MapTypeId.ROADMAP, disableDefaultUI: true, draggable: false, scrollwheel: false, styles: mapStyles};
		this.map = new google.maps.Map(this.mapContainer.html, mapOptions);

		var bounds = new google.maps.LatLngBounds();

		for (var i=0; i<locations.length; i++) {
			var l = locations[i];
			var lngLat = new google.maps.LatLng(l.latitude, l.longitude);
			var marker = new AREMapMarker(this.map, l.information.color, lngLat);

			marker.markerId = l.id;
			this.markers.push(marker);

			google.maps.event.addListener(marker, 'click', $.proxy(this.displayPopup, this));

			bounds.extend(lngLat);
		}
		//only run the map bounds if there is more than one location
		if (locations.length > 1) {
			this.map.fitBounds(bounds);
		}

		setTimeout($.proxy(this.fixMapMarkersNotClickable, this), 500);
	}
	// else {
	// 	console.log("No Google no fun");

	// 	var n = new ARENotificationView({heading:"CONNECTION ISSUE",body:"You are not connected to the internet, maps are not available."});
	// 	AREModalView.sharedInstance.displayNotification(n);
	// }
}

// Returns a decimal number, giving the amount to correct in the Y axis
// for the marker not to be centered in the map, to prevent cropping popups
AREMapContentView.prototype.mapCorrection = function(zoom) {
	if (zoom <= 3)  return 18.0;
	if (zoom == 4)  return 10.0;
	if (zoom == 5)  return  4.5;
	if (zoom == 6)  return  2.1;
	if (zoom == 7)  return  1.2;
	if (zoom == 8)  return  0.7;
	if (zoom == 9)  return  0.4;
	if (zoom == 10) return  0.15;
	if (zoom == 11) return  0.07;
	if (zoom == 12) return  0.04;
	if (zoom == 13) return  0.02;
	if (zoom == 14) return  0.01;
	if (zoom == 15) return  0.005;
	if (zoom == 16) return  0.0025;
	if (zoom == 17) return  0.0015;
	if (zoom == 18) return  0.0007;
	if (zoom == 19) return  0.0003;
	if (zoom == 20) return  0.00015;
	if (zoom >= 21) return  0.00008;
}

AREMapContentView.prototype.displayPopup = function(obj) {
	var id = obj.markerId;
	if (this.currentPopupId === null || id != this.currentPopupId) {
		var location = ApplicationData.sharedInstance.getLocationById(id);
		if (this.popup) this.popup.setMap(null);
		this.popup = new AREPopupView(this.map, obj.coords, location.information, {x:0, y:obj.offset.y-5}, true);
		$(this.popup).bind(AREPopupViewEvent.CLOSE_POPUP, $.proxy(this.closePopup, this));
		// $(this.popup).bind(AREPopupViewEvent.DISPAY_DIRECTIONS, $.proxy(this.displayDirections, this));
		// $(this.popup).bind(AREPopupViewEvent.VIEW_MORE, $.proxy(this.navigateToArticle, this));

		var lngLat = new google.maps.LatLng(location.latitude + this.mapCorrection(this.map.zoom), location.longitude);
		this.map.setCenter(lngLat);
		this.currentPopupId = id;

		//this.addDirections(obj.coords);
	}
	// else {
	// 	this.currentPopupId = null;
	// 	if (this.popup) {
	// 		$(this.popup).unbind();
	// 		this.popup.setMap(null);
	// 		this.popup = null;
	// 	}
	// }
}

// AREMapContentView.prototype.navigateToArticle = function(e,d) {
// 	$(this).trigger(AREMapContentViewEvent.VIEW_MORE, [d]);
// }

AREMapContentView.prototype.closePopup = function() {
	this.currentPopupId = null;
	if (this.popup) {
		$(this.popup).unbind();
		this.popup.setMap(null);
		this.popup = null;
	}
}

// AREMapContentView.prototype.displayDirections = function() {
// 	if (this.currentPopupId !== null && this.popup) {
// 		var location = ApplicationData.sharedInstance.getLocationById(this.currentPopupId);
// 		var userCoords = User.sharedInstance.location;

// 		// console.log("User.sharedInstance.location = "); console.log(userCoords);

// 		var toLat = location.latitude;
// 		var toLng = location.longitude;

// 		if (userCoords) {
// 			var fromLat = userCoords.lat();
// 			var fromLng = userCoords.lng();

// 			var url = "https://maps.google.co.uk/maps?saddr="+fromLat+","+fromLng+"&daddr="+toLat+","+toLng;
// 		} else {
// 			var url = "https://maps.google.co.uk/maps?daddr="+toLat+","+toLng;
// 		}

// 		window.open(url, '_blank', 'location=yes');

// 		// ga('send', 'pageview', "/Application/Map/Directions");
// 	}
// }