AREMapPageView.prototype = new AREView();
AREMapPageView.prototype.constructor = AREMapPageView;

function AREMapPageView(o) {
	this.dataObject = o;
	this.navigationBar = null;
	this.map = null;
	this.mapContainer = null;
	this.mapLoadedProxy = null;
	this.mapLoadedEventListener = null;
	this.homeMarker = null;
	this.coords = (this.dataObject.coords) ? this.dataObject.coords : null;
	this.targetButton = null;
	this.closeButton = null;
	this.filterButton = null;
	this.popup = null;
	this.currentPopupId = null;
	this.markers = new Array();
	this.ooeData = new Array();
	this.ooeMarkers = new Array();
	this.categoryData = this.dataObject.currentCategoryData;
	this.routePolylines = null;

	this.locations = (o.locations) ? o.locations : null;

	this.directionsDisplay = null;
	this.directionsService = (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) ? new google.maps.DirectionsService() : null;

	AREView.prototype.constructor.call(this);
}

AREMapPageView.prototype.fixMapMarkersNotClickable = function() {
	$('.gm-style').find('div[style*="z-index: 2"]').css('z-index','0');
}

AREMapPageView.prototype.drawView = function() {
	// console.log("Drawing Map view");

	if (googleAnalytics && navigator && navigator.onLine) googleAnalytics.trackPage("application/map/");

	AREView.prototype.drawView.call(this);
	this.addViewClass("mapPageView");

	this.mapLoadedProxy = $.proxy(this.mapLoadedEventHandler, this);

	this.mapContainer = new AREView();
	this.mapContainer.addViewClass("mapContainer");
	this.addSubview(this.mapContainer);

	this.navigationBar = new AREView();
	this.navigationBar.addViewClass("mapContentNavigationBar");
	this.addSubview(this.navigationBar);

	this.targetButton = new AREButton();
	this.targetButton.addViewClass("mapButtonLeft");
	var icon = new AREView();
	icon.addViewClass("targetIcon");
	this.targetButton.addSubview(icon);
	this.navigationBar.addSubview(this.targetButton);
	$(this.targetButton.html).click($.proxy(this.targetButtonPressed, this));

	this.closeButton = new AREButton();
	this.closeButton.addViewClass("mapButtonRight");
	var icon = new AREView();
	icon.addViewClass("closeIcon");
	this.closeButton.addSubview(icon);
	this.navigationBar.addSubview(this.closeButton);
	$(this.closeButton.html).click($.proxy(this.closeButtonPressed, this));

	this.filterButton = new AREButton();
	this.filterButton.addViewClass("mapButtonLeft");
	var icon = new AREView();
	icon.addViewClass("filterIcon");
	this.filterButton.addSubview(icon);
	this.navigationBar.addSubview(this.filterButton);
	$(this.filterButton.html).click($.proxy(this.filterButtonPressed, this));

	//setTimeout($.proxy(this.addMap, this), 200);
	setTimeout($.proxy(this.getLocation, this), 200);
}

AREMapPageView.prototype.getLocation = function() {
	if (navigator) {
		if (navigator.geolocation) navigator.geolocation.getCurrentPosition($.proxy(this.assignLocation, this), $.proxy(this.locationError, this));
	} else {
		this.addMap();
	}
}

AREMapPageView.prototype.locationError = function() {
	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {

		if (!this.coords) this.coords = new google.maps.LatLng(51.392458, -2.363777);
		this.addMap();

	} else {
		console.log("No Google no fun");

		var n = new ARENotificationView({heading:"CONNECTION ISSUE",body:"You are not connected to the internet, maps are not available."});
		AREModalView.sharedInstance.displayNotification(n);
	}
}

AREMapPageView.prototype.assignLocation = function(position) {
	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {

		User.sharedInstance.location = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
		if (!this.coords) this.coords = new google.maps.LatLng(position.coords.latitude, position.coords.longitude);
		this.addMap();

	} else {
		console.log("No Google no fun");

		var n = new ARENotificationView({heading:"CONNECTION ISSUE",body:"You are not connected to the internet, maps are not available."});
		AREModalView.sharedInstance.displayNotification(n);
	}
}

AREMapPageView.prototype.setCenter = function(c) {
	// console.log("setCenter: "); // console.log(c);
	var coords = c;
	if (coords) {
		this.map.setCenter(coords);
	}
}

AREMapPageView.prototype.targetButtonPressed = function() {
	// console.log("targetButtonPressed");
	if (this.map) {
		this.map.setCenter(User.sharedInstance.location);
	}
}

AREMapPageView.prototype.filterButtonPressed = function() {
	// console.log("filterButtonPressed");
	$(this).trigger(AREMapPageViewEvent.FILTER_BUTTON_SELECTED);
}

AREMapPageView.prototype.closeButtonPressed = function() {
	// console.log("closeButtonPressed");
	$(this).trigger(AREMapPageViewEvent.CLOSE_BUTTON_SELECTED);
}

AREMapPageView.prototype.addMap = function() {
	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {

		var mapStyles =[{
			featureType: "poi",
			elementType: "labels",
			stylers: [{ visibility: "off" }]
		}];
		var mapOptions = { zoom: 10, center: this.coords, mapTypeId: google.maps.MapTypeId.ROADMAP, disableDefaultUI: true, draggable:true, scrollwheel:false, styles:mapStyles };
		this.map = new google.maps.Map(this.mapContainer.html, mapOptions);
		this.mapLoadedEventListener = google.maps.event.addListener(this.map, "tilesloaded", this.mapLoadedProxy);

		if (User.sharedInstance.location) {
			this.homeMarker = new AREMapMarker(this.map, "#41d69c", User.sharedInstance.location, "homeMarkerImage", {x:-15,y:-20});
			google.maps.event.addListener(this.homeMarker, 'click', function(obj) {
			    //alert("top:"+obj.div.style.top+", left:"+obj.div.style.left);
			});
		}

		var bounds = new google.maps.LatLngBounds();

		// console.log("addMap function: CONTROL 1");
		var markersNum = 0;

		for (var i=0; i<ApplicationData.sharedInstance.locations.length; i++) {
			var location = ApplicationData.sharedInstance.locations[i];
			// console.log("addMap function, LOCATION:"); // console.log(location);

			var shouldDisplayMarker = true;
			if (this.locations) {
				if (this.locations.indexOf(location.id) < 0) {
					shouldDisplayMarker = false;
					// console.log(" -- location already in the markers");
				}
			}

			if (shouldDisplayMarker) {
				markersNum++;

				var lngLat = new google.maps.LatLng(location.latitude, location.longitude);
				var marker = new AREMapMarker(this.map, location.information.color, lngLat);
				marker.markerId = location.id;
				this.markers.push(marker);

				google.maps.event.addListener(marker, 'click', $.proxy(this.displayPopup, this));

				bounds.extend(lngLat);
			}
		}

		if (ApplicationData.sharedInstance.locations.length > 1) {
			console.log(bounds);
			if (markersNum > 1) {
				this.map.fitBounds(bounds);
			} else {
				if (markersNum == 1) {
					console.log(lngLat);
					this.map.setCenter(lngLat);
				}
			}
		}

		setTimeout($.proxy(this.fixMapMarkersNotClickable, this), 500);

		// if (this.coords) {
		// 	// console.log("Setting map centre to: "); // console.log(this.coords);
		// 	//this.map.setCenter(this.coords);
		// }
	} else {
		console.log("No Google no fun");

		var n = new ARENotificationView({heading:"CONNECTION ISSUE",body:"You are not connected to the internet, maps are not available."});
		AREModalView.sharedInstance.displayNotification(n);
	}
}

AREMapPageView.prototype.displayPopup = function(obj) {
	var id = obj.markerId;																						// console.log(obj);
	if (this.currentPopupId === null || id != this.currentPopupId) {
		var location = ApplicationData.sharedInstance.getLocationById(id);
		if (this.popup) this.popup.setMap(null);
		this.popup = new AREPopupView(this.map, obj.coords, location.information, {x:0, y:obj.offset.y-5});		// console.log(this.popup);
		$(this.popup).bind(AREPopupViewEvent.CLOSE_POPUP, $.proxy(this.closePopup, this));
		// $(this.popup).bind(AREPopupViewEvent.DISPAY_DIRECTIONS, $.proxy(this.displayDirections, this));
		$(this.popup).bind(AREPopupViewEvent.VIEW_MORE, $.proxy(this.navigateToArticle, this));
		this.map.setCenter(obj.coords);
		this.currentPopupId = id;

		if (User.sharedInstance.location) this.addDirections(obj.coords);
	}
	// else {
	// 	this.currentPopupId = null;
	// 	if (this.popup) {
	// 		$(this.popup).unbind();
	// 		this.popup.setMap(null);
	// 		this.popup = null;
	// 	}
	// }
}

AREMapPageView.prototype.navigateToArticle = function(e, d) {
	// console.log(d);
	$(this).trigger(AREMapPageViewEvent.VIEW_MORE_SELECTED, [d]);
}

AREMapPageView.prototype.closePopup = function() {
	this.currentPopupId = null;
	if (this.popup) {
		$(this.popup).unbind();
		this.popup.setMap(null);
		this.popup = null;
	}
	if (this.routePolylines) this.routePolylines.setMap(null);
}

AREMapPageView.prototype.addDirections = function(c) {

	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {

		/*this.directionsDisplay = new google.maps.DirectionsRenderer({
			draggable:false,
			suppressMarkers:true
		});
		this.directionsDisplay.setMap(this.map);*/
		if (this.routePolylines) this.routePolylines.setMap(null);

		var start = User.sharedInstance.location;
		var end = c;
		var request = {
			origin:start,
			destination:end,
			travelMode: google.maps.DirectionsTravelMode.DRIVING
		};
		this.directionsService.route(request, $.proxy(this.directionSuccess, this));
	} else {
		console.log("No Google no fun");

		var n = new ARENotificationView({heading:"CONNECTION ISSUE",body:"You are not connected to the internet, maps are not available."});
		AREModalView.sharedInstance.displayNotification(n);
	}
}

AREMapPageView.prototype.directionSuccess = function(response, status) {

	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {

		if (status == google.maps.DirectionsStatus.OK) {
			//if (this.directionsDisplay) this.directionsDisplay.setDirections(response);
			var points = new Array();
			var bounds = new google.maps.LatLngBounds();

			var route = response.routes[0].legs[0];
		        for (var i = 0; i < route.steps.length; i++) {
		        	for (var j = 0; j < route.steps[i].lat_lngs.length; j++) {
		        		points.push(route.steps[i].lat_lngs[j]);
		        		bounds.extend(route.steps[i].lat_lngs[j]);
		        	}
		        }

		        this.routePolylines = new google.maps.Polyline(
		        {
		            path: points,
		            strokeColor:"#2d9eee",
		            strokeOpacity: 0.6,
		            strokeWeight: 6,
		            clickable:false
		        });
			this.routePolylines.setMap(this.map);
			this.map.fitBounds(bounds);
		}
	} else {
		console.log("No Google no fun");

		var n = new ARENotificationView({heading:"CONNECTION ISSUE",body:"You are not connected to the internet, maps are not available."});
		AREModalView.sharedInstance.displayNotification(n);
	}
}

AREMapPageView.prototype.clearAllMarkers = function() {

	if (this.map) {
		for (var i=0; i<this.markers.length; i++) {
			this.markers[i].setMap(null);
		}
		this.markers = new Array();

		for (var i=0; i<this.ooeMarkers.length; i++) {
			this.ooeMarkers[i].setMap(null);
		}
		this.ooeMarkers = new Array();

		if (this.popup) this.popup.setMap(null);
	}
}

AREMapPageView.prototype.hideMarkerWithId = function(id) {

	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {

		if (this.map) {
			for (var i=0; i<this.markers.length; i++) {
				google.maps.event.clearListeners(this.markers[i], 'click');
				if (this.markers[i].markerId == id) this.markers[i].setMap(null);
			}
			if (this.popup) this.popup.setMap(null);
		}
	}
}

AREMapPageView.prototype.showMarkerWithId = function(id) {

	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {

		if (this.map) {
			/*for (var i=0; i<this.markers.length; i++) {
				if (this.markers[i].markerId == id) this.markers[i].setMap(this.map);
			}*/
			var location = ApplicationData.sharedInstance.getLocationById(id);
			if (location) {
				var lngLat = new google.maps.LatLng(location.latitude, location.longitude);
				var marker = new AREMapMarker(this.map, location.information.color, lngLat);
				marker.markerId = location.id;
				this.markers.push(marker);

				google.maps.event.addListener(marker, 'click', $.proxy(this.displayPopup, this));
			}
		}
	}
}

AREMapPageView.prototype.mapLoadedEventHandler = function() {

	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {

		// console.log("mapLoadedEventHandler called");
		google.maps.event.removeListener(this.mapLoadedEventListener);
		setTimeout($.proxy(function() { $(this).trigger(AREMapPageViewEvent.MAP_LOADED); }, this), 500);

		// console.log("Control point 1");
		//this.loadOoeData();
		//this.displayOoeData();
	}
}

AREMapPageView.prototype.loadOoeData = function() {

	var url = ApplicationData.sharedInstance.ooeData;
	var jsonLoader = $.getJSON(url+"?"+new Date().getTime());
	jsonLoader.done($.proxy(this.ooeDataLoaded, this));
}

AREMapPageView.prototype.ooeDataLoaded = function(data) {

	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {

		//this.ooeData = data.locations118["Pizza Suppliers"].Bath;

		// console.log("Control point 2");
		for (var d in this.ooeData) {
			var obj = this.ooeData[d];

			var lngLat = new google.maps.LatLng(obj.latitude, obj.longitude);
			var marker = new AREMapMarker(this.map, "#ffffff", lngLat, "ooeImage");
			marker.markerId = obj.id118;
			this.markers.push(marker);

			google.maps.event.addListener(marker, 'click', $.proxy(this.displayOoePopup, this));
		}
	}
}

AREMapPageView.prototype.clearOoeData = function() {

	for (var i=0; i<this.ooeMarkers.length; i++) {
		this.ooeMarkers[i].setMap(null);
	}
	this.ooeMarkers = new Array();
}

AREMapPageView.prototype.displayOoeData = function(t,a) {

	if (typeof google === 'object' && typeof google.maps === 'object' && navigator && navigator.onLine) {

		var ooeLocations = new Array();
		var ooeObj = ApplicationData.sharedInstance.ooeData.locations118;

		this.clearOoeData();

		if (t) {
			if (a) {
				for (var tObjKey in ooeObj) {
					if (t.indexOf(tObjKey) >= 0) {
						for (var aObjKey in ooeObj[tObjKey]) {
							if (a.indexOf(aObjKey) >=0 ) {
								var tObj = ooeObj[tObjKey];
								for (var lObjKey in tObj[aObjKey]) {
									var aObj = tObj[aObjKey];
									var lObj = aObj[lObjKey];
									ooeLocations.push(lObj);
								}
							}
						}
					}
				}
			} else {
				for (var tObjKey in ooeObj) {
					if (t.indexOf(tObjKey) >= 0) {
						for (var aObjKey in ooeObj[tObjKey]) {
							var tObj = ooeObj[tObjKey];
							for (var lObjKey in tObj[aObjKey]) {
								var aObj = tObj[aObjKey];
								var lObj = aObj[lObjKey];
								ooeLocations.push(lObj);
							}
						}
					}
				}
			}
		} else {
			if (a) {
				for (var tObjKey in ooeObj) {
					for (var aObjKey in ooeObj[tObjKey]) {
						if (a.indexOf(aObjKey) >=0 ) {
							var tObj = ooeObj[tObjKey];
							for (var lObjKey in tObj[aObjKey]) {
								var aObj = tObj[aObjKey];
								var lObj = aObj[lObjKey];
								ooeLocations.push(lObj);
							}
						}
					}
				}
			} else {
				for (var tObjKey in ooeObj) {
					for (var aObjKey in ooeObj[tObjKey]) {
						var tObj = ooeObj[tObjKey];
						for (var lObjKey in tObj[aObjKey]) {
							var aObj = tObj[aObjKey];
							var lObj = aObj[lObjKey];
							ooeLocations.push(lObj);
						}
					}
				}
			}
		}

		for (var i=0; i<ooeLocations.length; i++) {
			var obj = ooeLocations[i];

			var shouldAddLocation = true;
			if (this.categoryData && !this.isLocationListed(obj)) shouldAddLocation = false;

			if (shouldAddLocation) {
				this.ooeData.push(obj);

				var lngLat = new google.maps.LatLng(obj.latitude, obj.longitude);
				var marker = new AREMapMarker(this.map, "#ffffff", lngLat, "ooeImage");
				marker.markerId = obj.id118; // console.log("Binding displayer for: "+obj.id118);
				marker.ooeData = obj;
				this.ooeMarkers.push(marker);

				google.maps.event.addListener(marker, 'click', $.proxy(this.displayOoePopup, this));
			}
		}
	}
}

AREMapPageView.prototype.isLocationListed = function(obj) {

	var listed = false;
	if (this.categoryData) {
		if (this.categoryData.locations118) {
			// console.log(this.categoryData.locations118);
			// console.log(obj.id118);
			if (this.categoryData.locations118.indexOf(obj.id118) >= 0) {
				listed = true;
			}
		}
	}

	return listed;
}

AREMapPageView.prototype.displayOoePopup = function(obj) {

	var id = obj.markerId; // console.log("Displaying id: "+obj.markerId+". OBJ id: "+obj.ooeData.id118);
	// console.log(JSON.stringify(obj));
	if (this.currentPopupId === null || id != this.currentPopupId) {
		//var location = ApplicationData.sharedInstance.getLocationById(id);
		var info = new Object();
		info.heading = obj.ooeData.businessName;
		info.body = obj.ooeData.telephone;

		if (this.popup) this.popup.setMap(null);
		this.popup = new AREOoePopupView(this.map, obj.coords, info, {x:0, y:obj.offset.y-5});
		$(this.popup).bind(AREPopupViewEvent.CLOSE_POPUP, $.proxy(this.closePopup, this));
		// $(this.popup).bind(AREPopupViewEvent.DISPAY_DIRECTIONS, $.proxy(this.displayOoeDirections, this));
		this.map.setCenter(obj.coords);
		this.currentPopupId = id;
	}
	// else {
	// 	this.currentPopupId = null;
	// 	if (this.popup) {
	// 		$(this.popup).unbind();
	// 		this.popup.setMap(null);
	// 		this.popup = null;
	// 	}
	// }
}

